import numpy as np
import cv2 as cv
import importlib

Colors = importlib.import_module("greyBlobDetector", "Colors").Colors
GreyBlobDetector = importlib.import_module("greyBlobDetector", "GreyBlobDetector").GreyBlobDetector


class ContourAreaDetector:

    # HSV value for red
    RED_THRESHOLD = 8

    # HSV value for blue
    BLUE_THRESHOLD = 230

    def __init__(self):
        pass

    def detect_color(self, image, gaussian_blur = False):
        
        # gaussian blur is basically there to reduce the noise
        if gaussian_blur:
            # apply gaussian blur here
            image = cv.GaussianBlur(image,(5,5),0)

        gd = GreyBlobDetector()
        contours = gd.detect_color(image)

        if contours == None:
            return Colors.GREY
        else:
            # since the picture with the blobs is black and white, it can act as a mask
            # we also know that its either red or blue, so we can just check whether its colser to red
            # use the area since it would lessen the intergerance of neibouring pixels
            mask = np.zeros(image.shape[0], image.shape[1], 1) # mask m x n x 1
            # mask with all the points
            cv.drawContours(mask, contours, 0, (255), cv.FILLED)
            # applies the mask to image
            image[mask != 0] = (0,0,0)
            # transform this to hsv
            hsv = cv.cvtColor(image, cv.COLOR_BGR2HSV)
            # note : 0,0,0 will still be 0,0,0
            colors = hsv[:,:,0]
            average = colors.sum() / (colors != 0).sum()
            if abs(average - ContourAreaDetector.RED_THRESHOLD) \
                < abs(average - ContourAreaDetector.BLUE_THRESHOLD):
                return Colors.RED
            else:
                return Colors.BLUE
