import numpy as np
import cv2 as cv
import matplotlib.pyplot as plt
# reaplce with math.pi
PI = 3.141592653589793238

class Colors:
    RED = "RED"
    BLUE = "BLUE"
    GREY = "GREY"

def debug(image):
    cv.imshow("debug", image)
    cv.waitKey(0)
def debug_contours(contours, image):
    mask = np.zeros((image.shape[0], image.shape[1]))
    cv.drawContours(mask, contours, -1, (255), 1)
    cv.imshow("debug", mask)
    cv.waitKey(0)

class GreyBlobDetector:
    # we know that if an armor is either blue or red, it must be white except on the edges
    # since the shape does not change, we can try to find white blobs with the desired features

    # threshold to be considered white; 0 = black 255 = white, TBD
    # based on a rather dim image, might change if there's an algorithm to calculate this
    WHITE_THRESHOLD = 70
    
    # threshold for circularity (circle = 1); TBD
    # circularity = 4 * PI * AREA / (PERIMETER ^ 2)
    CIRCLE_THRESHOLD = 0.55 # based on test values of 0.72, 0.8, 0.68

    # threshold for concavity; the formula is area_of_blob / area_convexity_hull, TBD
    CONVEX_THRESHOLD = 0.85 # based on values around 0.95 and 0.98

    # threshold for how elongated it is (called intertia in opencv), TBD (circle = 1)
    MIN_INERTIA = 1.1
    MAX_INERTIA = 3.5

    # these parameters are for the shape of the image, not sure if needed or not
    WIDTH_RATIO = 0.2 # width LED / width image, TBD
    HEIGHT_RATIO = 0.67 # height LED / height image, TBD
    DIST_LEDS_RATIO = 0.67 # distance of the 2 LEDs / width of image, TBD

    MIN_AREA = None # to be initialized to 0.02 * area of image

    def __init__(self):
        pass

    def calculate_max_area(self, image):
        # this basically calcultes the min / max area 
        # false by default unless the ai that finds the armors is consistant enough with the
        # picture horizontal and vertical ratio
        x, y = image.shape
        return (x * GreyBlobDetector.WIDTH_RATIO)*(y * GreyBlobDetector.HEIGHT_RATIO) 

    def calculate_min_area(self, image):
        # needed here becase part of the vertical bars may be hidden
        return self.calculate_max_area(image) / 1.5 # TBD

    def calculate_distance(self, image):
        # only use this if there are 2 or more blobs detected
        x, y = image.shape
        return y * GreyBlobDetector.DIST_LEDS_RATIO # TBD

    # finds the contours of the LEDs and returns them
    def find_contours(self, image, area = False, distance = False, already_grayscale = False):
        # transform the image into black and white
        # first go to grayscale then goto black white with a threshold
        GreyBlobDetector.MIN_AREA = image.shape[0] * image.shape[1] * 0.02
        grey = image
        if not already_grayscale:
            grey = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
        # 0 = noir; 255 = blanc; threshold = 180
        # use HSV instead of cv.threshold
        black_white = cv.threshold(grey, GreyBlobDetector.WHITE_THRESHOLD, 255, cv.THRESH_BINARY)[1]
        # do find countours on the black white image
        contours, hierarchy = cv.findContours(black_white, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
        # note : other versions of OpenCV will require image, contours, hierarchy
        # now for the characteristics of the contours

        retained_contours = []
        for contour in contours:
            # TODO : something wrong here, probably with area
            area = cv.contourArea(contour)
            if area >= GreyBlobDetector.MIN_AREA:
                # circularity < threshold : is the contour NOT circular?
                circularity = 4*PI*area / (cv.arcLength(contour, True) * cv.arcLength(contour, True))
                # convexity > threashold : we want the shape to be convex
                convexity = area / cv.contourArea(cv.convexHull(contour))
                # opencv docs aren't very helpful on this but looks like inertia is aspect ratio
                (x,y),(width,height),theta = cv.minAreaRect(contour)
                # this is basically aspect ratio
                inertia = width / height
                # note : the angle is pretty much useless

                print("calculated circularity : ", circularity)
                print("threshold  circularity : ", GreyBlobDetector.CIRCLE_THRESHOLD)
                print()
                print("calculated converxity  : ", convexity)
                print("threshold  converxity  : ", GreyBlobDetector.CONVEX_THRESHOLD)
                print()
                print("calculated inertia     : ", inertia)
                print("threshold  inertia     : ", GreyBlobDetector.MIN_INERTIA, GreyBlobDetector.MAX_INERTIA)
                print()
                print("minimal area           : ", GreyBlobDetector.MIN_AREA)
                print("calculated area        : ", area)                
                # cv.SimpleBlobDetector
                if circularity > GreyBlobDetector.CIRCLE_THRESHOLD and \
                    convexity > GreyBlobDetector.CONVEX_THRESHOLD and \
                    inertia > GreyBlobDetector.MIN_INERTIA and \
                    inertia < GreyBlobDetector.MAX_INERTIA:
                    retained_contours.append(contour)
                    print("retained !")
                else:
                    print("rejected !")
                print("----------------------------------")

                # use HSV instead
                # 1. color = red or blue
                # use the other 2 to see if its gray
        return retained_contours

if __name__ == '__main__':
    image = cv.imread("test6.PNG")
    gbd = GreyBlobDetector()
    print(len(gbd.find_contours(image)))
